#!/usr/bin/env python
# -*- coding: utf-8 -*-
##################################################################
#
# Copyright (c) 2023 CICV, Inc. All Rights Reserved
#
##################################################################
"""
@Authors:           zhanghaiwen, yangzihao
@Data:              2024/02/21
@Last Modified:     2024/02/21
@Summary:           The template of custom indicator.
"""

"""
设计思路：

"""

import math
import pandas as pd
import numpy as np
from common import zip_time_pairs, continuous_group, get_status_active_data, _cal_THW, _cal_v_ego_projection
from log import logger

"""import functions"""


# custom metric codes
class CustomMetric(object):
    def __init__(self, all_data, case_name):
        self.data = all_data
        self.optimal_dict = self.data.config
        self.status_trigger_dict = self.data.status_trigger_dict
        self.case_name = case_name
        self.markline_df = pd.DataFrame(columns=['start_time', 'end_time', 'start_frame', 'end_frame', 'type'])
        self.graph_list = []

        self.df = pd.DataFrame()
        self.ego_df = pd.DataFrame()
        self.df_acc = pd.DataFrame()

        self.peak_time_THW = None

        self.result = {
            "name": "跟车峰值时间",
            "value": [],
            # "weight": [],
            "tableData": {
                "avg": "",  # 平均值，或指标值
                "max": "",
                "min": ""
            },
            "reportData": {
                "name": "跟车峰值时间（s）",
                # "legend": [], # 如果有多个data，则需要增加data对应的说明，如：["横向加速度", "纵向加速度"]
                "data": [],
                "markLine": [],
                "range": [],
            },
            "statusFlag": {}
        }
        self.run()
        print(f"跟车峰值时间: {self.result['value']}")

    def data_extract(self):
        self.df = self.data.object_df
        self.ego_df = self.data.ego_data

        active_time_ranges = self.status_trigger_dict['ACC']['ACC_active_time']
        self.df_acc = get_status_active_data(active_time_ranges, self.df)
        # self.df_ica = self.df[self.df['ACC_status'] == "Shut_off"].copy()  # 数字3对应ICA的Active
        # self.df_ica = self.df[self.df['ACC_status'] == "Active"].copy()  # 数字3对应ICA的Active

        if self.df_acc.empty:
            self.result['statusFlag']['function_ACC'] = False
        else:
            self.result['statusFlag']['function_ACC'] = True

    def _get_first_change_index_THW(self):
        """
        获取DataFrame中'set_headway_time'列首次发生变化的索引值。

        Args:
            无参数。

        Returns:
            Union[int, None]: 如果存在变化，则返回首次发生变化的索引值（int类型），否则返回None。

        """
        ego_x = self.df_acc[self.df_acc['playerId'] == 1]['posX'].reset_index(drop=True)
        ego_y = self.df_acc[self.df_acc['playerId'] == 1]['posY'].reset_index(drop=True)
        obj_x = self.df_acc[self.df_acc['playerId'] == 2]['posX'].reset_index(drop=True)
        obj_y = self.df_acc[self.df_acc['playerId'] == 2]['posY'].reset_index(drop=True)

        ego_speedx = self.df_acc[self.df_acc['playerId'] == 1]['speedX'].reset_index(drop=True)
        ego_speedy = self.df_acc[self.df_acc['playerId'] == 1]['speedY'].reset_index(drop=True)
        obj_speedx = self.df_acc[self.df_acc['playerId'] == 2]['speedX'].reset_index(drop=True)
        obj_speedy = self.df_acc[self.df_acc['playerId'] == 2]['speedY'].reset_index(drop=True)

        dx = obj_x - ego_x
        dy = obj_y - ego_y
        # vx = obj_speedx - ego_speedx
        # vy = obj_speedy - ego_speedy
        dist = np.sqrt(dx ** 2 + dy ** 2)
        ego_v_projection_in_dist = _cal_v_ego_projection(dx, dy, ego_speedx, ego_speedy)
        thw1 = _cal_THW(dist, ego_v_projection_in_dist)
        thw = thw1.tolist()
        THW = []
        for item in thw:
            THW.append(item)
            THW.append(item)
        self.df_acc['THW'] = THW

        change_indices = self.df_acc[self.df_acc['set_headway_time'] != self.df_acc['set_headway_time'].shift()].index
        if not change_indices.empty:
            first_change_index = change_indices.min()
        else:
            first_change_index = None
        return first_change_index

    def data_analyze(self):
        first_change_index = self._get_first_change_index_THW()

        if not first_change_index:
            self.peak_time_THW = 0
            self.result['value'] = [round(self.peak_time_THW, 3)]
            print(f"peak_time_THW: {self.peak_time_THW}")
        else:
            start_time = self.df_acc.loc[first_change_index, 'simTime']
            peak_time = self.df_acc.loc[self.df_acc['THW'].idxmax(), 'simTime']
            self.peak_time_THW = peak_time - start_time
            self.result['value'] = [round(self.peak_time_THW, 3)]
            print(f"peak_time_THW: {self.peak_time_THW}")

    def markline_statistic(self):
        pass

    def report_data_statistic(self):
        # time_list = self.ego_df['simTime'].values.tolist()
        # graph_list = [x for x in self.graph_list if not np.isnan(x)]
        self.result['tableData']['avg'] = self.result['value'][0] if not self.df_acc.empty else '-'
        self.result['tableData']['max'] = '-'
        self.result['tableData']['min'] = '-'

        # zip_vs_time = zip_time_pairs(time_list, self.graph_list)
        self.result['reportData']['data'] = []

        # self.markline_statistic()
        # markline_slices = self.markline_df.to_dict('records')
        self.result['reportData']['markLine'] = []

        self.result['reportData']['range'] = [0, 1.2]

    def run(self):
        # logger.info(f"Custom metric run:[{self.result['name']}].")
        logger.info(f"[case:{self.case_name}] Custom metric:[peak_time_THW:{self.result['name']}] evaluate.")

        try:
            self.data_extract()
        except Exception as e:
            logger.error(f"[case:{self.case_name}] Custom metric:{self.result['name']} data extract ERROR!", e)

        try:
            self.data_analyze()
        except Exception as e:
            logger.error(f"[case:{self.case_name}] Custom metric:{self.result['name']} data analyze ERROR!", e)

        try:
            self.report_data_statistic()
        except Exception as e:
            logger.error(f"[case:{self.case_name}] Custom metric:{self.result['name']} report data statistic ERROR!", e)

# if __name__ == "__main__":
#     pass
